﻿' Projekteinstellungen:
' Option Strict On
' Option Explicit On
' Option Infer On
' Imports System.Windows.Forms

Imports System.IO

Public Class frmMain

    Private _PrevMouse As New MouseEventArgs(0, 0, 0, 0, 0)

#Region "DragDrop empfangen"

    ' Im _DragOver muß durch Setzen von e.Effect festgelegt werden, welcher 
    '  DropEffect momentan unterstützt wird. Hier kommt nur .Copy in Frage, da 
    '  .Move die Source-Datei ja löschen würde. Bei Nicht-Festlegen gilt 
    '  DragDropEffects.None, und es kommt nie zum Dropping
    Private Sub ListBox1_DragOver(ByVal sender As Object, _
        ByVal e As DragEventArgs) Handles ListBox1.DragOver

        Dim DTO = TryCast(e.Data, DataObject)
        If DTO IsNot Nothing AndAlso DTO.ContainsFileDropList Then
            e.Effect = DragDropEffects.Copy
        End If
    End Sub

    ' Im _DragDrop wird nach dem schlußendlich gültigen Drop-Effect verfahren. 
    ' Dieses Beispiel braucht aber nicht auf verschiedene DropEffekte 
    ' abzuprüfen, da nur DragDropEffects.Copy möglich ist.
    Private Sub ListBox1_DragDrop(ByVal sender As Object, _
        ByVal e As DragEventArgs) Handles ListBox1.DragDrop

        Dim DTO = TryCast(e.Data, DataObject)
        ListBox1.Items.AddRange(DTO.GetFileDropList.Cast(Of String).ToArray)
    End Sub

#End Region 'DragDrop empfangen

#Region "DragDrop senden"
    ' Dragging wird gestartet, wenn ein Mousmove mit gehaltener Links-Taste 
    ' einem Links-Tasten-MouseDown folgt.
    Private Sub ListBox1_MouseDown(ByVal sender As Object, _
        ByVal e As MouseEventArgs) Handles ListBox1.MouseDown

        _PrevMouse = e
    End Sub

    Private Sub ListBox1_MouseMove(ByVal sender As Object, _
        ByVal e As MouseEventArgs) Handles ListBox1.MouseMove

        If _PrevMouse.Button = MouseButtons.Left AndAlso _
            e.Button = MouseButtons.Left Then

            Dim I = ListBox1.IndexFromPoint(_PrevMouse.Location)
            If I < 0 Then Return

            Dim SC As New System.Collections.Specialized.StringCollection
            Dim FullPath = ListBox1.Items(I).ToString
            SC.Add(FullPath)

            Dim DTO = New DataObject
            DTO.SetFileDropList(SC)

            ListBox1.AllowDrop = False
            Dim Result = ListBox1.DoDragDrop(DTO, _
                DragDropEffects.Copy Or DragDropEffects.Move)
            ListBox1.AllowDrop = True

            Select Case Result
                Case DragDropEffects.Copy
                    ' Nichts tun. Kopieren muß die Anwendung, auf die 
                    ' gedroppt wurde

                Case DragDropEffects.Move
                    ' Dieses funktioniert leider nicht.
                    ' Bei DropEffekt.Move bekomme ich DropEffekt.None zurück
                    ListBox1.Items.RemoveAt(I)

                Case DragDropEffects.None
                    ' Also gucken wir, ob das FilesystemItem gelöscht wurde
                    If (Not File.Exists(FullPath)) AndAlso _
                            (Not Directory.Exists(FullPath)) Then
                        ListBox1.Items.RemoveAt(I)
                    End If

            End Select
        End If
        _PrevMouse = e
    End Sub

#End Region 'DragDrop senden

End Class